/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: ooxcontexthelper.hxx,v $
 *
 *  $Revision: 1.2.4.2 $
 *
 *  last change: $Author: dr $ $Date: 2008/01/25 14:24:05 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef OOX_XLS_OOXCONTEXTHELPER_HXX
#define OOX_XLS_OOXCONTEXTHELPER_HXX

#include <vector>
#include <map>
#include <boost/shared_ptr.hpp>
#include <rtl/ustrbuf.hxx>
#include <com/sun/star/uno/Reference.hxx>
#include "oox/core/recordcontext.hxx"
#include "oox/xls/ooxtokens.hxx"

namespace com { namespace sun { namespace star {
    namespace xml { namespace sax {
        class XFastAttributeList;
        class XFastContextHandler;
    } }
} } }

namespace oox { class AttributeList; }

namespace oox {
namespace xls {

// ============================================================================

class OoxContextHelper;
typedef ::rtl::Reference< OoxContextHelper > OoxContextHelperRef;

// ============================================================================

/** Wrapper for a context handler supporting XFastContextHandler.

    This wrapper is used by the OoxContextHelper class to simplify returning
    either the own context handler instance, a new context handler, or null to
    indicate an unsupported context. The implicit constructors allow to return
    either a C++ bool value (true = own context instance, false = null) or a
    new context handler (any class derived from OoxContextHelper, can be null
    too) in the implementation of the virtual OoxContextHelper::onCreateContext()
    function. This wrapper takes ownership of the passed context handler
    internally, thus it is possible to simply pass a new heap-allocated object.
 */
class OoxContextWrapper
{
public:
    inline /*implicit*/ OoxContextWrapper( bool bThis ) : mbThis( bThis ) {}
    inline /*implicit*/ OoxContextWrapper( OoxContextHelper* pContext ) : mxContext( pContext ), mbThis( false ) {}

    inline OoxContextHelper* getContext( OoxContextHelper* pThis ) const
                            { return mbThis ? pThis : mxContext.get(); }

private:
    OoxContextHelperRef mxContext;
    bool                mbThis;
};

// ============================================================================

struct OoxContextInfo;

/** Helper class that provides a context identifier stack.

    Fragment handlers and context handlers derived from this helper class will
    track the identifiers of the current context in a stack. The idea is to use
    the same instance of a fragment handler or context handler to process
    several nested elements in a stream. For that, the abstract function
    onCreateContext() has to return 'true' for the passed element.

    Derived classes have to implement the createFastChildContext(),
    startFastElement(), characters(), and endFastElement() functions from the
    com.sun.star.xml.sax.XFastContextHandler interface by simply forwarding
    them to the respective implCreateChildContext(), implStartCurrentContext(),
    implCharacters(), and implEndCurrentContext() functions of this helper. The
    new abstract functions have to be implemented according to the elements to
    be processed.

    Similarly, for binary import, derived classes have to forward the
    createRecordContext(), startRecord(), and endRecord() functions from the
    oox.core.RecordContext interface to the implCreateRecordContext(),
    implStartRecord(), and implEndRecord() functions of this helper.
 */
class OoxContextHelper
{
public:
    virtual void SAL_CALL acquire() throw() = 0;
    virtual void SAL_CALL release() throw() = 0;

protected:
    explicit            OoxContextHelper();
    virtual             ~OoxContextHelper();

    // interface --------------------------------------------------------------

    /** Used to receive the actual XFastContextHandler interface from the
        derived class.
     */
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastContextHandler >
                        queryContextHandler() = 0;

    /** Will be called if a new context can be created for the passed element.

        Usually 'true' should be returned to improve performance by reusing the
        same instance to process several elements. Used by OOXML import only.
     */
    virtual OoxContextWrapper onCreateContext( sal_Int32 nElement, const AttributeList& rAttribs ) = 0;

    /** Will be called if a new context element has been started.

        The current element identifier can be accessed by using
        getCurrentContext() or isCurrentContext(). Used by OOXML import only.
     */
    virtual void        onStartElement( const AttributeList& rAttribs ) = 0;

    /** Will be called if the current context element is about to be left.

        The current element identifier can be accessed by using
        getCurrentContext() or isCurrentContext(). Used by OOXML import only.

        @param rChars  The characters collected in this element.
     */
    virtual void        onEndElement( const ::rtl::OUString& rChars ) = 0;

    /** Used to receive the actual oox.core.RecordContext interface from the
        derived class.
     */
    virtual ::oox::core::RecordContextRef queryRecordContext() = 0;

    /** Will be called if a new context can be created for the passed element.

        Usually 'true' should be returned to improve performance by reusing the
        same instance to process several elements. Used by OOBIN import only.
     */
    virtual OoxContextWrapper onCreateRecordContext( sal_Int32 nRecId, RecordInputStream& rStrm ) = 0;

    /** Will be called if a new record in a binary stream has been started.

        The current record identifier can be accessed by using
        getCurrentContext() or isCurrentContext(). Used by OOBIN import only.
     */
    virtual void        onStartRecord( RecordInputStream& rStrm ) = 0;

    /** Will be called if the current context record is about to be left.

        The current record identifier can be accessed by using
        getCurrentContext() or isCurrentContext(). Used by OOBIN import only.
     */
    virtual void        onEndRecord() = 0;

    // helpers ----------------------------------------------------------------

    /** Returns the element identifier of the current topmost context. */
    sal_Int32           getCurrentContext() const;

    /** Returns true, if nElement contains the current topmost context. */
    inline bool         isCurrentContext( sal_Int32 nElement ) const
                            { return getCurrentContext() == nElement; }

    /** Returns true, if either nElement1 or nElement2 contain the current topmost context. */
    inline bool         isCurrentContext( sal_Int32 nElement1, sal_Int32 nElement2 ) const
                            { return isCurrentContext( nElement1 ) || isCurrentContext( nElement2 ); }

    /** Returns the element identifier of the specified parent context. */
    sal_Int32           getPreviousContext( sal_Int32 nCountBack = 1 ) const;

    /** Returns the element identifier of the specified parent context. */
    inline sal_Int32    isPreviousContext( sal_Int32 nElement, sal_Int32 nCountBack = 1 ) const
                            { return getPreviousContext( nCountBack ) == nElement; }

    /** Must be called from createFastChildContext() in derived classes. */
    ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastContextHandler >
                        implCreateChildContext(
                            sal_Int32 nElement,
                            const ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastAttributeList >& rxAttribs );

    /** Must be called from startFastElement() in derived classes. */
    void                implStartCurrentContext(
                            sal_Int32 nElement,
                            const ::com::sun::star::uno::Reference< ::com::sun::star::xml::sax::XFastAttributeList >& rxAttribs );

    /** Must be called from characters() in derived classes. */
    void                implCharacters( const ::rtl::OUString& rChars );

    /** Must be called from endFastElement() in derived classes. */
    void                implEndCurrentContext( sal_Int32 nElement );

    /** Must be called from createRecordContext() in derived classes. */
    ::oox::core::RecordContextRef
                        implCreateRecordContext( sal_Int32 nRecId, RecordInputStream& rStrm );

    /** Must be called from startRecord() in derived classes. */
    void                implStartRecord( sal_Int32 nRecId, RecordInputStream& rStrm );

    /** Must be called from endRecord() in derived classes. */
    void                implEndRecord( sal_Int32 nRecId );

private:
    OoxContextInfo&     pushContextInfo( sal_Int32 nElement );
    void                popContextInfo();
    void                appendCollectedChars();

private:
    typedef ::std::vector< OoxContextInfo >         OoxContextStack;
    typedef ::boost::shared_ptr< OoxContextStack >  OoxContextStackRef;

    OoxContextStackRef  mxContextStack;     /// Stack of processed contexts.
};

// ============================================================================

} // namespace xls
} // namespace oox

#endif

